<?php

/**
 * MyBBIntegrator - The integration class for MyBB and your website
 *
 * The MyBBIntegrator is a useful collection of variables and functions for easy MyBB integration
 * into the own website
 *
 * So far the MyBBIntegrator includes following features:
 * - dbEscape: Escape Values for Database usage
 * - getForum: Returns forum data of $forum_id
 * - getLatestActiveThreads: Get latest active threads, where posts have been posted
 * - getLatestPosts: Get latest posts of one or more forums
 * - getLatestThreads: Get latest threads of one or more forums
 * - getPost: Returns post data of $post_id, also parse it instantly
 * - getPosts:
 * - getPostsOfThread: Return posts from a specific thread
 * - getPrivateMessagesOfUser: Get messages of a user in a nicely structured array
 * - getThread: Returns thread data of $thread_id
 * - getUser: Returns user data of $user_id
 * - getWhoIsOnline: Returns list of online users
 * - incViews: Increase the view count of a thread
 * - isLoggedIn: Return status if the user is logged in
 * - isModerator: Return status if the user is a mod
 * - isSuperAdmin: Returns if user is super admin
 * - login: Login Procedure with user and password
 * - logout: Logout Procedure
 * - markRead: Mark one or more forums read
 * - parseString: Parses a string with MyBB's parser (MyCodes/BBCodes)
 * - register: Register a new user to the forum. User Info is taken from an array passed to the function
 *
 * @author: David Olah (aka PHPDave - http://phpdave.com)
 * @version 1.1
 * @copyright Copyright (c) 2009, David Olah
 *
 *
 *
 * 	This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

class MyBBIntegrator
{	
	/**
	 * Cache Handler of MyBB
	 *
	 * @var object
	*/
	var $cache;
	
	/**
	 * Database Handler of MyBB
	 *
	 * @var object
	*/
	var $db;
	
	/**
	 * MyBB Super Variable containing a whole lot of information
	 *
	 * @var object
	*/
	var $mybb;
	
	/**
	 * Path of the forum, NO TRAILING SLASH
	 *
	 * @var array
	*/
	var $mybb_path = '../forums/MyBB_1.4.4';
	
	/**
	 * MyBB's Post Parser
	 *
	 * @var object
	*/
	var $parser;
	
	/**
	 * Shows a message for errors occuring in this class. 
	 * Afterwards it stops the script
	 *
	 * @param string $message The error message
	*/
	function _errorAndDie($message)
	{
		echo '<div style="width:92%; margin:4px auto; border:1px #DDD solid; background:#F1F1F1; padding:5px; color:#C00; font-weight:bold;">An error occured during script run.<br />'.$message.'</div>';
		die;
	}
	
	/**
	 * Escapes a value for DB usage
	 *
	 * @param mixed $value Any value to use with the database
	 * @return string
	*/
	function dbEscape($value)
	{
		return $this->db->escape_string($value);
	}
	
	/**
	 * Returns data of a specified forum
	 * Refers to: inc/functions.php
	 *
	 * @param integer $forum_id ID of forum to fetch data from
	 * @param integer $active_override If set to 1, will override the active forum status
	 * @return array|boolean If unsuccessful, it returns false - Otherwise the Database row
	*/
	function getForum($forum_id, $active_override = 0)
	{
		$forum = get_forum($forum_id, $active_override);
		
		// Do we have permission?
		$forumpermissions = forum_permissions($forum['fid']);
		if ($forumpermissions['canview'] != 1 || $forumpermissions['canviewthreads'] != 1)
		{
			// error_no_permission();
			return false;
		}
		else
		{
			return $forum;
		}
	}
	
	/**
	 * Return the latest threads of one forum, where a post has been posted
	 *
	 * @param integer $forum_id Forum ID to fetch threads from
	 * @param integer $limit Amount of threads to get
	 * @param boolean $excluse_invisible Shall we also get invisible threads?
	 * @return array
	*/
	function getLatestActiveThreads($forum_id = 0, $limit = 7, $exclude_invisible = true)
	{
		if ($forum_id == 0)
		{
			$this->_errorAndDie('Specified forum ID cannot be 0!');
		}
		else
		{
			// Do we have permission?
			$forumpermissions = forum_permissions($forum_id);
			if ($forumpermissions['canview'] != 1 || $forumpermissions['canviewthreads'] != 1)
			{
				// error_no_permission();
				return falsse;
			}
		}
		
		// This will be the array, where we can save the threads
		$threads = array();
		
		// We want to get a list of threads, starting with the newest one
		$query_params = array(
			'order_by' => 'lastpost',
			'order_dir' => 'DESC',
			'limit' => intval($limit)
		);
		
		/**
		 * If defined forum id is 0, we do not fetch threads from only one forum,
		 * but we fetch the latest threads of all forums
		 * Therefore we add the forum_id in the where condition
		 * We only fetch visible threads, if there is anything we want to hide ;)
		 * However we can also define that we want the invisible threads as well
		*/
		$fetch_invisible_threads = ($exclude_invisible == true) ? '1' : '0';
		$condition = ($forum_id != 0) ? ' `visible` = '.$fetch_invisible_threads.' AND `fid` = '.intval($forum_id) : '';
		
		// Run the Query
		$query = $this->db->simple_select('threads', '*', $condition, $query_params);
		
		// Now let's iterate through the fetched threads to create the return array
		while ($thread = $this->db->fetch_array($query))
		{
			$threads[] = $thread;
		}
		
		return $threads;
	}
	
	/**
	 * Return newly created threads, regardless of replies
	 *
	 * @param integer|array $forum_id Forum ID / Forum IDs to fetch threads from
	 * @param string $fields Name of fields if you want to fetch specific fields
	 * @param integer $limit Amount of threads to get
	 * @param boolean $excluse_invisible Shall we also get invisible threads?
	 * @param boolean $join_forums Shall we also get the information from the forums where the threads are located in?
	 * @param boolean $join_first_post Shall we get the first post of this thread as well?
	 * @return array
	*/
	function getLatestThreads($forum_id = 0, $fields = '*', $limit = 7, $exclude_invisible = true, $join_forums = true, $join_first_post = true)
	{
		if ($forum_id != 0)
		{
			// Do we have permission?
			$forumpermissions = forum_permissions($forum_id);
			if ($forumpermissions['canview'] != 1 || $forumpermissions['canviewthreads'] != 1)
			{
				// error_no_permission();
				return false;
			}
		}
		
		// This is what we will be returning
		$threads = array();
		
		// Do we want to get invisible threads as well?
		$fetch_invisible_threads = ($exclude_invisible == true) ? '1' : '0';
		$condition = 't.`visible` = '.$fetch_invisible_threads;
		
		// Are we fetching threads from multiple forums?
		if (is_array($forum_id) || is_object($forum_id))
		{
			$condition .= ' AND t.`fid` IN ('.implode(', ', $forum_id).')';
		}
		// Or are we just fetching threads from one forum?
		else
		{
			$condition .= ($forum_id == 0) ? '' : ' AND t.`fid` = '.$forum_id;
		}
		
		// Do we want to get information of the forum where the thread is located in?
		$forum_join = ($join_forums == true) ? 'INNER JOIN '.TABLE_PREFIX.'forums f ON f.`fid` = t.`fid`' : '';
		
		// Do we want to get the first post from the thread?
		$first_post_join = ($join_first_post == true) ? 'INNER JOIN '.TABLE_PREFIX.'posts p ON p.`pid` = t.`firstpost`' : '';
		
		// Run the Query
		$query = $this->db->query('
			SELECT '.$fields.'
			FROM '.TABLE_PREFIX.'threads t
			'.$forum_join.'
			'.$first_post_join.'
			WHERE '.$condition.'
			ORDER BY t.`dateline` DESC
			LIMIT '.intval($limit).'
		');
		
		// Iterate through the results and assign it to our returning array
		while ($thread = $this->db->fetch_array($query))
		{
			$threads[] = $thread;
		}
		
		return $threads;
	}
	
	/**
	 * Return recently posted posts
	 *
	 * @param integer|array Either a single Thread ID or an array containing thread IDs
	 * @param string Fields, which shall be fetched from the posts table
	 * @param integer How many posts shall be fetched?
	 * @param boolean Shall we also return invisible ones?
	 * @return array
	*/
	function getLatestPosts($thread_id = 0, $fields = '*', $limit = 7, $exclude_invisible = true)
	{		
		// Posts will be stored in this array
		$posts = array();
		
		// Posts will be returned in descending order, starting with the newest
		$query_params = array(
			'order_by' => 'dateline',
			'order_dir' => 'DESC',
			'limit' => intval($limit)
		);
		
		// We want to fetch posts from multiple threads
		if (is_array($thread_id) || is_object($thread_id))
		{
			// Multiple threads = IN (...) Operator
			$condition = '`fid` IN ('.implode(', ', $thread_id).')';
		}
		else
		{
			// Single thread = normal WHERE X = Y - if set 0 we fetch posts from all threads
			$condition = ($thread_id == 0) ? '' : '`fid` = '.intval($thread_id);
		}
		
		/**
		 * If defined forum id is 0, we do not fetch threads from only one forum,
		 * but we fetch the latest threads of all forums
		 * Therefore we add the forum_id in the where condition
		 * We only fetch visible threads, if there is anything we want to hide ;)
		 * However we can also define that we want the invisible threads as well
		*/
		$fetch_invisible_threads = ($exclude_invisible == true) ? '1' : '0';
		$condition .= ' AND `visible` = '.$excluse_invisible;
		
		// Run the Query
		$query = $this->db->simple_select('posts', $fields, $condition, $query_params);
		
		// Now let's iterate through the fetched posts to create the return array
		while ($post = $this->db->fetch_array($query))
		{
			$posts[] = $post;
		}
		
		return $posts;
	}
	
	/**
	 * Returns post data of specified post
	 * Refers to: inc/functions.php & inc/class_parser.php
	 *
	 * @param integer $post_id Post ID to fetch data from
	 * @param boolean $parsed Shall the Post message be parsed?
	 * @param array $parse_options Array of yes/no options - allow_html,filter_badwords,allow_mycode,allow_smilies,nl2br,me_username
	 * @param array $override_forum_parse_options Whether parse options should be defined by forum or by the script.
	 												If they are being overridden, the array will contain the options
	 * @return array|boolean: If unsuccessful, it returns false - Otherwise the Database row
	*/
	function getPost($post_id, $parsed = false, $override_forum_parse_options = array())
	{
		if ($post_id == 0)
		{
			$this->_errorAndDie('Specified post ID cannot be 0!');
		}
		
		// Get the Post data
		$post = get_post($post_id);
		
		// Post not found? --> False
		if (empty($post))
		{
			return false;
		}
		
		// Do we have permission?
		$forumpermissions = forum_permissions($post['fid']);
		if ($forumpermissions['canview'] != 1 || $forumpermissions['canviewthreads'] != 1)
		{
			// error_no_permission();
			return false;
		}
		
		// If the post shall not be parsed, we can already return it at this point
		if ($parsed == false || empty($post))
		{
			return $post;
		}
		
		// So we want to parse the message
		
		/**
		 * We don't want to override the parse options defined by the forum,
		 * so we have first to get these options defined for the forum
		*/
		if (count($override_forum_parse_options) == 0)
		{
			// Get the Forum data according to the forum id stored with the post
			$forum = $this->getForum($post['fid']);
			
			// Set up the parser options.
			$parser_options = array(
				"allow_html" => $forum['allowhtml'],
				"allow_mycode" => $forum['allowmycode'],
				"allow_smilies" => $forum['allowsmilies'],
				"allow_imgcode" => $forum['allowimgcode'],
				"filter_badwords" => 1
			);
			
		}
		else
		{
			// Self-defined options given in the function parameter
			$parser_options = array(
				'allow_html' => (isset($override_forum_parse_options['allow_html']) && $override_forum_parse_options['allow_html'] == 1) ? 1 : 0,
				'allow_mycode' => (isset($override_forum_parse_options['allow_mycode']) && $override_forum_parse_options['allow_mycode'] == 1) ? 1 : 0,
				'allow_smilies' => (isset($override_forum_parse_options['allow_smilies']) && $override_forum_parse_options['allow_smilies'] == 1) ? 1 : 0,
				'allow_imgcode' => (isset($override_forum_parse_options['allow_imgcode']) && $override_forum_parse_options['allow_imgcode'] == 1) ? 1 : 0,
				'filter_badwords' => (isset($override_forum_parse_options['filter_badwords']) && $override_forum_parse_options['filter_badwords'] == 1) ? 1 : 0,
			);
		}
		
		// Overwrite the message with the parsed message
		$post['message'] = $this->parser->parse_message($post['message'], $parser_options);
		
		return $post;
	}
	
	/**
	 * Get posts which match the given criteria
	 *
	 * @param array $params Parameters for the query
	 * @return array
	*/
	function getPosts($params = array('fields' => '*', 'order_by' => 'dateline', 'order_dir' => 'DESC', 'limit_start' => 0, 'limit' => 0, 'where' => ''))
	{
		// We will store the posts in here
		$posts = array();
		
		// No matter what parameters will be given, the query starts with the following
		$sql = 'SELECT '.$params['fields'].'
			    FROM '.TABLE_PREFIX.'posts';
		
		// Get all posts or just (hopefully) posts which match certain criteria?
		$sql .= ($params['where'] != '') ? ' WHERE '.$params['where'] : '';
		
		// Are the posts going to be ordered by a field?
		if ($params['order_by'] != '')
		{
			$sql .= ' ORDER BY '.$params['order_by'];
			if ($params['order_dir'] != '')
			{
				$sql .= ' '.$params['order_dir'];
			}
			else
			{
				$sql .= ' ASC';
			}
		}
		
		// Get all posts or (hopefully) just a few?
		if ($params['limit'] != 0)
		{
			$sql .= ' LIMIT ';
			if (isset($params['limit_start']))
			{
				$sql .= $params['limit_start'].', '.$params['limit'];
			}
			else
			{
				$sql .= $params['limit'];
			}
		}
		
		// Run the query
		$query = $this->db->query($sql);
		
		// Store the returned data in the array we return
		while ($post = $this->db->fetch_array($query))
		{
			$posts[] = $post;
		}
		
		return $posts;
	}
	
	/**
	 * Get the Posts of a particular thread
	 *
	 * @param integer $thread_id
	 * @param string $fields If you want to fetch certain fields, define them as a string here (separated by comma)
	 * @param array $options Options for the query [ array('limit_start', 'limit', 'orderby', 'order_dir') ]
	 * @return array
	*/
	function getPostsOfThread($thread_id, $fields = '*', $options = array())
	{
		// This is what we will be returning
		$arr = array();
		
		$thread = $this->db->fetch_field('SELECT `fid` FROM '.TABLE_PREFIX.'threads WHERE `tid` = '.intval($thread_id).' LIMIT 1', 0);
		
		// Do we have permission?
		$forumpermissions = forum_permissions($thread['fid']);
		if ($forumpermissions['canview'] != 1 || $forumpermissions['canviewthreads'] != 1)
		{
			// error_no_permission();
			return false;
		}
		
		// Let's request the posts from the database
		$query = $this->db->simple_select('posts', $fields, '`tid` = '.intval($thread_id), $options);
		
		// All we need to do now is to assign them to our returning array
		while ($post = $this->db->fetch_array($query))
		{
			$arr[] = $post;
		}
		
		return $arr;
	}
	
	/**
	 * Read the messages from database of a user
	 *
	 * @param integer $user_id ID of user
	 * @param array $params Array with options for SQL Query (orderby, sort)
	 * @param boolean $translate_folders If the folders should be turned into readable format  la "inbox"
	 * @return array
	*/
	function getPrivateMessagesOfUser($user_id, $params = array('orderby' => 'pm.dateline', 'sort' => 'DESC'), $translate_folders = true)
	{		
		/**
		 * This is what we will be returning
		 * Structure of the array to return:
		 * array(
		 *    'Inbox' => array( ... Messages ... )
		 * )
		 *
		 * 'Inbox' is the translated folder of folder #1
		*/
		$arr = array();
		
		// If we want to translate the folder names, we need to include the file which contains the translation function
		if ($translate_folders == true)
		{
			include_once MYBB_ROOT.'inc/functions_user.php';
		}
		
		// Run the Query for Private Messages
		$query = $this->db->query('
			SELECT pm.*, fu.username AS fromusername, tu.username as tousername
			FROM '.TABLE_PREFIX.'privatemessages pm
			LEFT JOIN '.TABLE_PREFIX.'users fu ON (fu.uid=pm.fromid)
			LEFT JOIN '.TABLE_PREFIX.'users tu ON (tu.uid=pm.toid)
			WHERE pm.uid = '.intval($user_id).'
			ORDER BY '.$params['orderby'].' '.$params['sort'].'
		');
		
		// Do we have messages?
		if ($this->db->num_rows($query) > 0)
		{
			// Uhh, let's iterate the messages!
			while ($message = $this->db->fetch_array($query))
			{
				// If we translate the folder names, our array index will be the translated folder name
				if ($translate_folders == true)
				{
					$arr[get_pm_folder_name($message['folder'])][] = $message;
				}
				// If we don't want translated folder names, our array index will be the folder number
				else
				{
					$arr[$message['folder']][] = $message;
				}
			}
		}
		
		return $arr;
	}
	
	/**
	 * Returns data of a specified thread
	 * Refers to: inc/functions.php
	 *
	 * @param integer $thread_id ID of the thread to fetch data from
	 * @return array|boolean If unsuccessful, it returns false - Otherwise the Database row
	*/
	function getThread($thread_id)
	{
		$thread = get_thread($thread_id);
		
		// Do we have permission?
		$forumpermissions = forum_permissions($thread['fid']);
		if ($forumpermissions['canview'] != 1 || $forumpermissions['canviewthreads'] != 1)
		{
			// error_no_permission();
			return false;
		}
		else
		{
			return $thread;
		}
	}
	
	/**
	 * Get Threads of one or more forums
	 *
	 * @param integer $forum_id IDs of Forums to fetch threads from
	 * @param string $fields If you want to fetch certain fields, define a string with them
	 * @param string $where Additional WHERE constellation if needed
	 * @pararm array $query_params Parameters for the Query to run in the database 
	 							   (order_by, order_dir, limit_start, limit [limit will only be acknowledged if both limit vars are defined])
	 * @param boolean $excluse_invisible Shall we get invisible threads too?
	 * @param boolean $join_forums Do we also want to get the forum information of where the threads are located?
	 * @param boolean $join_first_post Shall we get the first post of the thread? (= initial post)
	 * @return array
	*/
	function getThreads($forum_id, $fields = '*', $where = '', $query_params = array('order_by' => 't.`subject`', 'order_dir' => 'ASC'), $exclude_invisible = true, $join_forums = false, $join_first_post = false)
	{
		// Do we have permission?
		if (is_array($forum_id))
		{
			$forumpermissions = forum_permissions($forum_id);
			if ($forumpermissions['canview'] != 1 || $forumpermissions['canviewthreads'] != 1)
			{
				// error_no_permission();
				return false;
			}
		}
		else
		{
			// Check for every single forum
			foreach ($forum_id as $forum_id_single)
			{
				$forumpermissions = forum_permissions($forum_id_single);
				if ($forumpermissions['canview'] != 1 || $forumpermissions['canviewthreads'] != 1)
				{
					// error_no_permission();
					return false;
				}
			}
		}
		
		// This is what we will be returning
		$threads = array();
		
		// Do we want to get invisible threads as well?
		$fetch_invisible_threads = ($exclude_invisible == true) ? '1' : '0';
		$condition = 't.`visible` = '.$fetch_invisible_threads;
		
		// Are we fetching threads from multiple forums?
		if (is_array($forum_id))
		{
			$condition .= ' AND t.`fid` IN ('.implode(', ', $forum_id).')';
		}
		// Or are we just fetching threads from one forum?
		else
		{
			$condition .= ($forum_id == 0) ? '' : ' AND t.`fid` = '.$forum_id;
		}
		
		// An additional WHERE clause has been added
		if ($where != '')
		{
			$condition .= ' AND '.$where;
		}
		
		// Do we want to get information of the forum where the thread is located in?
		$forum_join = ($join_forums == true) ? 'INNER JOIN '.TABLE_PREFIX.'forums f ON f.`fid` = t.`fid`' : '';
		
		// Do we want to get the first post from the thread?
		$first_post_join = ($join_first_post == true) ? 'INNER JOIN '.TABLE_PREFIX.'posts p ON p.`pid` = t.`firstpost`' : '';
		
		// Is a Limit defined?
		$limit = (isset($query_params['limit_start']) && isset($query_params['limit'])) ? 'LIMIT '.intval($query_params['limit_start']).', '.intval($query_params['limit']) : '';
		
		// Run the Query
		$query = $this->db->query('
			SELECT '.$fields.'
			FROM '.TABLE_PREFIX.'threads t
			'.$forum_join.'
			'.$first_post_join.'
			WHERE '.$condition.'
			ORDER BY '.$query_params['order_by'].' '.$query_params['order_dir'].'
			'.$limit.'
		');
		
		// Iterate through the results and assign it to our returning array
		while ($thread = $this->db->fetch_array($query))
		{
			$threads[] = $thread;
		}
		
		return $threads;
	}
	
	/**
	 * Returns data of a user
	 * Refers to: inc/functions.php
	 *
	 * @param integer $user_id ID of User to fetch data from (0 = own user)
	 * @return array
	*/
	function getUser($user_id = 0)
	{
		// If given user id is 0, we use the own User ID
		if ($user_id == 0)
		{
			return get_user($this->mybb->user['uid']);
		}
		// Otherwise we fetch info from given User ID
		else
		{
			return get_user($user_id);
		}
	}
	
	/**
	 * Fetch the users being online
	 * Refers to: index.php
	 *
	 * @param boolean $colored_usernames Define if we want to return formatted usernames (color)
	 * @return array
	*/
	function getWhoIsOnline($colored_usernames = true)
	{
		// This is what we are going to return
		$arr = array(
			'bots' => array(),
			'count_anonymous' => 0,
			'count_bots' => 0,
			'count_guests' => 0,
			'count_members' => 0,
			'members' => array()
		);
		
		// We only fetch the Who's Online list if the setting tells us that we can
		if($this->mybb->settings['showwol'] != 0 && $this->mybb->usergroup['canviewonline'] != 0)
		{
			// Get the online users.
			$timesearch = TIME_NOW - $this->mybb->settings['wolcutoff'];
			$comma = '';
			$query = $this->db->query("
				SELECT s.sid, s.ip, s.uid, s.time, s.location, s.location1, u.username, u.invisible, u.usergroup, u.displaygroup
				FROM ".TABLE_PREFIX."sessions s
				LEFT JOIN ".TABLE_PREFIX."users u ON (s.uid=u.uid)
				WHERE s.time>'$timesearch'
				ORDER BY u.username ASC, s.time DESC
			");
			
			// Iterated users will be stored here to prevent double iterating one user
			$doneusers = array();
		
			// Fetch spiders
			$spiders = $this->cache->read("spiders");
		
			// Loop through all users.
			while($user = $this->db->fetch_array($query))
			{				
				// Create a key to test if this user is a search bot.
				$botkey = my_strtolower(str_replace("bot=", '', $user['sid']));
		
				// Decide what type of user we are dealing with.
				if($user['uid'] > 0)
				{
					// The user is registered.
					if($doneusers[$user['uid']] < $user['time'] || !$doneusers[$user['uid']])
					{
						// If the user is logged in anonymously, update the count for that.
						if($user['invisible'] == 1)
						{
							++$arr['count_anonymous'];
						}
						
						if($user['invisible'] != 1 || $this->mybb->usergroup['canviewwolinvis'] == 1 || $user['uid'] == $this->mybb->user['uid'])
						{		
							// Maybe we don't want colored usernames
							if ($colored_usernames == true)
							{
								// Properly format the username and assign the template.
								$user['username'] = format_name($user['username'], $user['usergroup'], $user['displaygroup']);
							}

							$user['profilelink'] = build_profile_link($user['username'], $user['uid']);
						}
						
						// This user has been handled.
						$doneusers[$user['uid']] = $user['time'];
						
						// Add the user to the members, since he is registered and logged in
						$arr['members'][]= $user;
						
						// Increase member counter
						++$arr['count_members'];
					}
				}
				
				// The user is a search bot.
				elseif(my_strpos($user['sid'], "bot=") !== false && $spiders[$botkey])
				{
					++$arr['count_bots'];
					$arr['bots'][] = $spiders[$botkey];
				}
				
				// The user is a guest
				else
				{
					++$arr['count_guests'];
				}
			}
		}
		
		return $arr;
	}
	
	/**
	 * Increases the Amount of Views of a thread
	 *
	 * @param integer $thread_id ID of Thread
	*/
	function incViews($thread_id)
	{
		// All we do here is to run the increment query
		$this->db->query('
			UPDATE '.TABLE_PREFIX.'threads
			SET `views` = `views` + 1
			WHERE `tid` = '.intval($thread_id).'
		');
	}
	
	/**
	 * Is the user logged in?
	 *
	 * @return boolean
	*/
	function isLoggedIn()
	{
		// If the user is logged in, he has an UID
		return ($this->mybb->user['uid'] != 0) ? true : false;
	}
	
	/**
	 * Is the user a moderator?
	 * This function checks if the user has certain rights to perform an action in a forum
	 * Refers to: inc/functions.php
	 *
	 * @param integer $forum_id ID of the forum to check permissions for
	 * @param string $action Action which shall be checked for
	 * @param integer $user_id ID of User
	*/
	function isModerator($forum_id = 0, $action = '', $user_id = 0)
	{
		// If we aren't logged in, we cannot possibly a moderator
		if ($this->isLoggedIn())
		{
			return false;
		}
		
		// If given user_id is 0 we tak the user_id of the current user --> Check if own user is mod
		return is_moderator($forum_id, $action, ($user_id == 0) ? $this->mybb->user['uid'] : $user_id);
	}
	
	/**
	 * Returns if a user has super admin permission
	 * Refers to: inc/functions.php
	 *
	 * @param integer $user_id User-ID
	 * @return boolean
	*/
	function isSuperAdmin($user_id = 0)
	{
		// If specified user_id is 0, we want to know if current user is Super Admin
		return is_super_admin(($user_id == 0) ? $this->mybb->user['uid'] : $user_id);
	}
	
	/**
	 * Login procedure for a user + password
	 * Possible ToDo: Return error messages / array / whatever
	 *
	 * @param string $username Username
	 * @param string $password Password of User
	 * @return boolean
	*/
	function login($username, $password)
	{
		/**
		 * If we are already logged in, we do not have to perform the login procedure
		 * However, we can make believe that the login did succeed
		 * It certainly did a while ago ;)
		*/
		if ($this->isLoggedIn())
		{
			return true;
		}
		
		/**
		 * Let's see how many logins we have already tried
		 * Has no effect yet, but could be good for a ToDo
		*/
		$logins = login_attempt_check();
		
		// We need a few functions from the user function collection for the login procedur
		require_once MYBB_ROOT.'inc/functions_user.php';
		
		// If the username does not exist, login fails
		if(!username_exists($username))
		{
			my_setcookie('loginattempts', $logins + 1);
			return false;
		}
		
		/**
		 * Let's get a database version of the login attempts
		 * Previous login attempt call relied on cookies
		*/
		$query = $this->db->simple_select("users", "loginattempts", "LOWER(username)='".$this->db->escape_string(my_strtolower($username))."'", array('limit' => 1));
		$loginattempts = $this->db->fetch_field($query, "loginattempts");
		
		// Let's call the handy MyBB validation function and see if we find a user
		$user = validate_password_from_username($username, $password);
		if (!$user['uid'])
		{
			my_setcookie('loginattempts', $logins + 1);
			$this->db->write_query("UPDATE ".TABLE_PREFIX."users SET `loginattempts` = `loginattempts` + 1 WHERE LOWER(`username`) = '".$this->db->escape_string(my_strtolower($username))."'");
			return false;
		}
		
		/**
		 * Possible ToDo:
		 * If we have had more than 3 login attemps a captcha is shown in MyBB
		 * Maybe provide the same functionality in MyBBIntegrator ?
		*/
		
		// COPPA users always fail :D
		if ($user['coppauser'])
		{
			return false;
		}
		
		// Reset both login attempts counter (cookie + database)
		my_setcookie('loginattempts', 1);
		$this->db->update_query("users", array("loginattempts" => 1), "uid='{$user['uid']}'");
		
		// Delete old session entry
		$this->db->delete_query("sessions", "ip='".$this->db->escape_string($this->mybb->session->ipaddress)."' AND sid != '".$this->mybb->session->sid."'");
		
		// Create a new session and save it in the database
		$newsession = array(
			"uid" => $user['uid'],
		);
		$this->db->update_query("sessions", $newsession, "sid='".$this->mybb->session->sid."'");
		
		// Temporarily set the cookie remember option for the login cookies
		$this->mybb->user['remember'] = $user['remember'];
		
		// Set essential login cookies
		my_setcookie("mybbuser", $user['uid']."_".$user['loginkey'], null, true);
		my_setcookie("sid", $this->mybb->session->sid, -1, true);
		
		// If there are hooks defined for the end of the login procedure, call them
		$this->plugins->run_hooks("member_do_login_end");
		
		return true;
	}
	
	/**
	 * Logout procedure
	 *
	 * @return boolean
	*/
	function logout()
	{
		// If the user is not logged in at all, we make him believe that the logout procedure workedjust fine
		if (!$this->isLoggedIn())
		{
			return true;
		}
		
		// Check session ID if we have one
		if($this->mybb->input['sid'] && $this->mybb->input['sid'] != $this->mybb->session->sid)
		{
			return false;
		}
		// Otherwise, check logoutkey
		else if(!$this->mybb->input['sid'] && $this->mybb->input['logoutkey'] != $this->mybb->user['logoutkey'])
		{
			return false;
		}
		
		// Clear essential login cookies
		my_unsetcookie("mybbuser");
		my_unsetcookie("sid");
		
		// The logged in user data will be updated
		if($this->mybb->user['uid'])
		{
			$time = TIME_NOW;
			$lastvisit = array(
				"lastactive" => $time-900,
				"lastvisit" => $time,
			);
			$this->db->update_query("users", $lastvisit, "uid='".$this->mybb->user['uid']."'");
			$this->db->delete_query("sessions", "sid='".$this->mybb->session->sid."'");
		}
		
		// If there are any hooks to run, we call them here
		$this->plugins->run_hooks("member_logout_end");
		
		return true;
	}
	
	/**
	 * Marks one or more forums read
	 *
	 * @param integer $forum_id If Forum ID is set to 0 all forums will be marked as read
	 * @param string $redirect_url If there should be a redirection afterwards, define the URL here
	 * @return boolean Only returns false if it fails, otherwise, if it does not redirect, it returns true
	*/
	function markRead($id = 0, $redirect_url = '')
	{
		// "Mark-Read" functions are located in inc/functions_indicators.php
		require_once MYBB_ROOT."/inc/functions_indicators.php";
		
		// Make sure the ID is a number
		$id = intval($id);
		
		// If the given Forum ID is 0, it tells us that we shall mark all forums as read
		if ($id == 0)
		{
			mark_all_forums_read();
			
			// If we want to redirect to an url, we do so
			if ($redirect_url != '')
			{
				redirect($redirect_url, $this->lang->redirect_markforumsread);
			}
		}
		
		// If a specific ID has been defined, we certainly want to mark ONE forum as read
		else
		{
			// Does the Forum exist?
			$validforum = $this->getForum($id);
			
			// If the forum is invalid, marking as read failed
			if (!$validforum)
			{
				return false;
			}
			
			// If we want to redirect to an url, we do so
			if ($redirect_url != '')
			{
				redirect($redirect_url, $this->lang->redirect_markforumsread);
			}
		}
		
		return true;
	}
	
	/**
	 * Constructor
	 * If we include the global.php of MyBB in the constructor,
	 * weird error messages popup
	 * --> This sounds like a ToDo: Find out why?????? - My guess: Some evil eval() functions
	 *
	 * @param object $mybb Pass the Super MyBB Object as a reference so we can work with it
	 * @param object $db Pass the MyBB Database Handler as a reference so we can work with is
	*/
	function MyBBIntegrator(&$mybb, &$db, &$cache, &$plugins, &$lang)
	{
		$this->mybb =& $mybb;
		$this->db =& $db;
		$this->cache = $cache;
		$this->plugins =& $plugins;
		$this->lang =& $lang;
		
		require_once MYBB_ROOT.'inc/class_parser.php';
		$this->parser = new postParser;
	}
	
	/**
	 * Parses a string/message with the MyBB Parser Class
	 * Refers to: /inc/class_parser.php
	 *
	 * @param string $message The String which shall be parsed
	 * @param array $options Options the parser can accept: filter_badwords, allow_html, allow_mycode, me_username, allow_smilies, nl2br, 
	 * @return string Parsed string
	*/
	function parseString($message, $options = array())
	{
		// Set base URL for parsing smilies
		$base_url= $this->mybb->settings['bburl'];

		if($base_url!= "")
		{
			if(my_substr($base_url, my_strlen($base_url) -1) != "/")
			{
				$base_url= $base_url."/";
			}
		}

		$message = $this->plugins->run_hooks("parse_message_start", $message);

		// Get rid of cartridge returns for they are the workings of the devil
		$message = str_replace("\r", "", $message);

		// Filter bad words if requested.
		if($options['filter_badwords'])
		{
			$message = $this->parser->parse_badwords($message);
		}

		if($options['allow_html'] != 1)
		{
			$message = $this->parser->parse_html($message);
		}
		else
		{		
			while(preg_match("#<script(.*)>(.*)</script(.*)>#is", $message))
			{
				$message = preg_replace("#<script(.*)>(.*)</script(.*)>#is", "&lt;script$1&gt;$2&lt;/script$3&gt;", $message);
			}
			// Remove these completely
			$message = preg_replace("#\s*<base[^>]*>\s*#is", "", $message);
			$message = preg_replace("#\s*<meta[^>]*>\s*#is", "", $message);
			$message = str_replace(array('<?php', '<!--', '-->', '?>', "<br />\n", "<br>\n"), array('&lt;?php', '&lt;!--', '--&gt;', '?&gt;', "\n", "\n"), $message);
		}
		
		// If MyCode needs to be replaced, first filter out [code] and [php] tags.
		if($options['allow_mycode'])
		{
			// First we split up the contents of code and php tags to ensure they're not parsed.
			preg_match_all("#\[(code|php)\](.*?)\[/\\1\](\r\n?|\n?)#si", $message, $code_matches, PREG_SET_ORDER);
			$message = preg_replace("#\[(code|php)\](.*?)\[/\\1\](\r\n?|\n?)#si", "<mybb-code>\n", $message);
		}

		// Always fix bad Javascript in the message.
		$message = $this->parser->fix_javascript($message);
		
		// Replace "me" code and slaps if we have a username
		if($options['me_username'])
		{			
			$message = preg_replace('#(>|^|\r|\n)/me ([^\r\n<]*)#i', "\\1<span style=\"color: red;\">* {$options['me_username']} \\2</span>", $message);
			$message = preg_replace('#(>|^|\r|\n)/slap ([^\r\n<]*)#i', "\\1<span style=\"color: red;\">* {$options['me_username']} {$this->lang->slaps} \\2 {$this->lang->with_trout}</span>", $message);
		}
		
		// If we can, parse smilies
		if($options['allow_smilies'])
		{
			$message = $this->parser->parse_smilies($message, $options['allow_html']);
		}

		// Replace MyCode if requested.
		if($options['allow_mycode'])
		{
			$message = $this->parser->parse_mycode($message, $options);
		}

		// Run plugin hooks
		$message = $this->plugins->run_hooks("parse_message", $message);
		
		if($options['allow_mycode'])
		{
			// Now that we're done, if we split up any code tags, parse them and glue it all back together
			if(count($code_matches) > 0)
			{
				foreach($code_matches as $text)
				{
					// Fix up HTML inside the code tags so it is clean
					if($options['allow_html'] != 0)
					{
						$text[2] = $this->parser->parse_html($text[2]);
					}
					
					if(my_strtolower($text[1]) == "code")
					{
						$code = $this->parser->mycode_parse_code($text[2]);
					}
					elseif(my_strtolower($text[1]) == "php")
					{
						$code = $this->parser->mycode_parse_php($text[2]);
					}
					$message = preg_replace("#\<mybb-code>\n?#", $code, $message, 1);
				}
			}
		}

		if($options['nl2br'] !== 0)
		{
			$message = nl2br($message);
			// Fix up new lines and block level elements
			$message = preg_replace("#(</?(?:html|head|body|div|p|form|table|thead|tbody|tfoot|tr|td|th|ul|ol|li|div|p|blockquote|cite|hr)[^>]*>)\s*<br />#i", "$1", $message);
			$message = preg_replace("#(&nbsp;)+(</?(?:html|head|body|div|p|form|table|thead|tbody|tfoot|tr|td|th|ul|ol|li|div|p|blockquote|cite|hr)[^>]*>)#i", "$2", $message);
		}

		$message = my_wordwrap($message);
	
		$message = $this->plugins->run_hooks("parse_message_end", $message);
				
		return $message;
	}
	
	/**
	 * Register procedure
	 * Refers to: /member.php
	 *
	 * @param array $info Contains user information of the User to be registered
	 * @return array|string If registration fails, we return an array containing the error message, 
	 * 						If registration is successful, we return the string, which notifies the user of what will be the next action
	*/
	function register($info = array())
	{
		// Load the language phrases we need for the registration
		$this->lang->load('member');
		
		/**
		 * $info contains the given user information for the registration
		 * We need to make sure that every possible key is given, so we do not generate ugly E_NOIICE errors
		*/
		$possible_info_keys	= array(
			'username', 'password', 'password2', 'email', 'email2', 'referrer', 'timezone', 'language',
			'profile_fields', 'allownotices', 'hideemail', 'subscriptionmethod', 
			'receivepms', 'pmnotice', 'emailpmnotify', 'invisible', 'dstcorrection'
		);
		
		// Iterate the possible info keys to create the array entry in $info if it does not exist
		foreach ($possible_info_keys as $possible_info_key)
		{
			if (!isset($info[$possible_info_key]))
			{
				$info[$possible_info_key] = '';
			}
		}
		
		// Run whatever hook specified at the beginning of the registration		
		$this->plugins->run_hooks('member_do_register_start');
		
		// If register type is random password, we generate one
		if($this->mybb->settings['regtype'] == "randompass")
		{
			$info['password'] = random_str();
			$info['password2'] = $info['password'];
		}
		
		if($this->mybb->settings['regtype'] == "verify" || $this->mybb->settings['regtype'] == "admin" || $info['coppa'] == 1)
		{
			$usergroup = 5;
		}
		else
		{
			$usergroup = 2;
		}
		
		// Set up user handler.
		require_once MYBB_ROOT."inc/datahandlers/user.php";
		$userhandler = new UserDataHandler("insert");
		
		// Set the data for the new user.
		$user = array(
			"username" => $info['username'],
			"password" => $info['password'],
			"password2" => $info['password2'],
			"email" => $info['email'],
			"email2" => $info['email2'],
			"usergroup" => $usergroup,
			"referrer" => $info['referrername'],
			"timezone" => $info['timezoneoffset'],
			"language" => $info['language'],
			"profile_fields" => $info['profile_fields'],
			"regip" => $this->mybb->session->ipaddress,
			"longregip" => ip2long($this->mybb->session->ipaddress),
			"coppa_user" => intval($this->mybb->cookies['coppauser']),
		);
		
		if(isset($info['regcheck1']) && isset($info['regcheck2']))
		{
			$user['regcheck1'] = $info['regcheck1'];
			$user['regcheck2'] = $info['regcheck2'];
		}
		
		// Do we have a saved COPPA DOB?
		if($this->mybb->cookies['coppadob'])
		{
			list($dob_day, $dob_month, $dob_year) = explode("-", $this->mybb->cookies['coppadob']);
			$user['birthday'] = array(
				"day" => $dob_day,
				"month" => $dob_month,
				"year" => $dob_year
			);
		}
		
		// Generate the options array of the user
		$user['options'] = array(
			"allownotices" => $info['allownotices'],
			"hideemail" => $info['hideemail'],
			"subscriptionmethod" => $info['subscriptionmethod'],
			"receivepms" => $info['receivepms'],
			"pmnotice" => $info['pmnotice'],
			"emailpmnotify" => $info['emailpmnotify'],
			"invisible" => $info['invisible'],
			"dstcorrection" => $info['dstcorrection']
		);
		
		// Assign data to the data handler
		$userhandler->set_data($user);
		
		// If the validation of the user failed, we return nice (friendly) errors
		if(!$userhandler->validate_user())
		{
			$errors = $userhandler->get_friendly_errors();
			return $errors;
		}
		
		// Create the User in the database
		$user_info = $userhandler->insert_user();
		
		// We need to set a cookie, if we don't want a random password (and it is no COPPA user), so he is instantly logged in
		if($this->mybb->settings['regtype'] != "randompass" && !$this->mybb->cookies['coppauser'])
		{
			// Log them in
			my_setcookie("mybbuser", $user_info['uid']."_".$user_info['loginkey'], null, true);
		}
		
		/**
		 * Coppa User
		 * Nothing special, just return that the coppa user will be redirected
		*/
		if($this->mybb->cookies['coppauser'])
		{
			$this->lang->redirect_registered_coppa_activate = $this->lang->sprintf($this->lang->redirect_registered_coppa_activate, $this->mybb->settings['bbname'], $user_info['username']);
			my_unsetcookie("coppauser");
			my_unsetcookie("coppadob");
			
			// Run whatever hook is defined at the end of a registration
			$this->plugins->run_hooks("member_do_register_end");
			
			return $this->lang->redirect_registered_coppa_activate;
		}
		
		/**
		 * Register Mode: Email Verification
		 * A mail is dispatched containing an activation link.
		 * The activation link is a reference to the newly created database entry
		*/
		else if($this->mybb->settings['regtype'] == "verify")
		{
			// Generate and save the activation code in the database
			$activationcode = random_str();
			$now = TIME_NOW;
			$activationarray = array(
				"uid" => $user_info['uid'],
				"dateline" => TIME_NOW,
				"code" => $activationcode,
				"type" => "r"
			);
			$this->db->insert_query("awaitingactivation", $activationarray);
			
			// Generate and send the email
			$emailsubject = $this->lang->sprintf($this->lang->emailsubject_activateaccount, $this->mybb->settings['bbname']);
			$emailmessage = $this->lang->sprintf($this->lang->email_activateaccount, $user_info['username'], $this->mybb->settings['bbname'], $this->mybb->settings['bburl'], $user_info['uid'], $activationcode);
			my_mail($user_info['email'], $emailsubject, $emailmessage);
			
			// Build the message to return
			$this->lang->redirect_registered_activation = $this->lang->sprintf($this->lang->redirect_registered_activation, $this->mybb->settings['bbname'], $user_info['username']);
			
			// Run whatever hook is defined at the end of a registration
			$this->plugins->run_hooks("member_do_register_end");
			
			return $this->lang->redirect_registered_activation;
		}
		
		/**
		 * Register Mode: Send Random Password
		 * A mail is dispatched, containing the random password for the user
		*/
		else if($this->mybb->settings['regtype'] == "randompass")
		{
			// Generate and send the email
			$emailsubject = $this->lang->sprintf($this->lang->emailsubject_randompassword, $this->mybb->settings['bbname']);
			$emailmessage = $this->lang->sprintf($this->lang->email_randompassword, $user['username'], $this->mybb->settings['bbname'], $user_info['username'], $user_info['password']);
			my_mail($user_info['email'], $emailsubject, $emailmessage);
			
			// Run whatever hook is defined at the end of a registration
			$this->plugins->run_hooks("member_do_register_end");
			
			return $this->lang->redirect_registered_passwordsent;
		}
		
		/**
		 * Register Mode: Admin Activation
		 * Return the message that the user will need to be authorized by an admin
		*/
		else if($this->mybb->settings['regtype'] == "admin")
		{
			// Build the message to return
			$this->lang->redirect_registered_admin_activate = $this->lang->sprintf($this->lang->redirect_registered_admin_activate, $this->mybb->settings['bbname'], $user_info['username']);
			
			// Run whatever hook is defined at the end of a registration
			$this->plugins->run_hooks("member_do_register_end");
			
			return $this->lang->redirect_registered_admin_activate;
		}
		
		/**
		 * No activation required whatsoever,
		 * directly registered
		*/
		else
		{
			// Build the message to return
			$this->lang->redirect_registered = $this->lang->sprintf($this->lang->redirect_registered, $this->mybb->settings['bbname'], $user_info['username']);
			
			// Run whatever hook is defined at the end of a registration
			$this->plugins->run_hooks("member_do_register_end");
			
			return $this->lang->redirect_registered;
		}
	}
}

?>